package com.winhc.max.compute.graph.job.enterprise_group;

import com.aliyun.odps.data.TableInfo;
import com.aliyun.odps.graph.GraphJob;
import com.aliyun.odps.graph.RemoveDuplicatesLoadingResolver;
import com.winhc.max.compute.graph.job.pagerank.CompanyComputingVertexResolver;
import com.winhc.max.compute.graph.util.BaseUtils;
import com.winhc.max.compute.graph.util.DateUtils;
import com.winhc.max.compute.graph.util.ParameterTool;

import java.io.IOException;

/**
 * @author: XuJiakai
 * 2023/2/7 13:58
 */
public class EnterpriseGroupJob {


    private static final String defaultInputVertexTab;
    private static final String defaultEdgeInputTab;
    private static final String defaultOutputParentTab;
    private static final String defaultOutputMainTab = "calc_enterprise_group_main_tab";
    private static final String defaultOutputRelationTab = "calc_enterprise_group_relation_tab";

    static {
        if (BaseUtils.isWindows()) {
            defaultInputVertexTab = "tmp_xjk_test_enterprise_group_input_vertex";
            defaultEdgeInputTab = "tmp_xjk_test_enterprise_group_input_edge";
            defaultOutputParentTab = "tmp_xjk_test_enterprise_group_out";
        } else {
            defaultInputVertexTab = "calc_enterprise_group_input_vertex";
            defaultEdgeInputTab = "calc_enterprise_group_input_edge";
            defaultOutputParentTab = "calc_enterprise_group_out_parent_tab";
        }
    }


    public static void main(String[] args) throws IOException {
        String nowDate = DateUtils.nowDate(DateUtils.YYYYMMDDHHMMSS).replace(" ", ".").replace(":", "");

        ParameterTool parameterTool = ParameterTool.fromArgs(args);
        String numWorkers = parameterTool.getOrDefault("numWorkers", "70");
        String workerCPU = parameterTool.getOrDefault("workerCPU", "2");
        String workerMem = parameterTool.getOrDefault("workerMem", "8192");

        String inputVertexTab = parameterTool.getOrDefault("inputVertexTab", defaultInputVertexTab);
        String edgeInputTab = parameterTool.getOrDefault("edgeInputTab", defaultEdgeInputTab);
        String outputParentTab = parameterTool.getOrDefault("outputParentTab", defaultOutputParentTab);
        String outputMainTab = parameterTool.getOrDefault("outputMainTab", defaultOutputMainTab);
        String outputRelationTab = parameterTool.getOrDefault("outputRelationTab", defaultOutputRelationTab);

        String outPart = parameterTool.getOrDefault("outPart", nowDate);
        String debugVertexId = parameterTool.getOrDefault("debugVertexId", null);
        System.out.println("input args: " + String.join(" ", args));
        GraphJob job = new GraphJob();


        job.setMaxIteration(-1);
        job.setNumWorkers(Integer.parseInt(numWorkers));
        if (parameterTool.has("workerCPU")) {
            job.setWorkerCPU(Integer.parseInt(workerCPU) * 100);
        }
        if (parameterTool.has("workerMem")) {
            job.setWorkerMemory(Integer.parseInt(workerMem));
        }

        if (parameterTool.has("enableDebug")) {
            String debugIndex = parameterTool.getOrDefault("debugIndex", null);
            job.set("winhc.debug", "true");
            job.set("winhc.debug.index", debugIndex);
            job.set("winhc.debug.debugVertexId", debugVertexId);
        } else {
            job.set("winhc.debug", "false");
        }


        job.setGraphLoaderClass(EnterpriseGroupReader.class);
        job.setVertexClass(EnterpriseGroupVertex.class);


        job.setLoadingVertexResolver(RemoveDuplicatesLoadingResolver.class);
        job.setComputingVertexResolver(CompanyComputingVertexResolver.class);
        job.setAggregatorClass(EnterpriseGroupAggregator.class);


        job.addInput(TableInfo.builder()
                .projectName("winhc_ng")
                .label("vertex")
                .tableName(inputVertexTab)
                .build());
        job.addInput(TableInfo.builder()
                .projectName("winhc_ng")
                .tableName(edgeInputTab)
                .label("edge")
                .build());

        job.addOutput(TableInfo.builder()
                .projectName("winhc_ng")
                .tableName(outputParentTab)
                .label("parent")
                .partSpec("ds='" + outPart + "'")
                .build());
        job.addOutput(TableInfo.builder()
                .projectName("winhc_ng")
                .tableName(outputMainTab)
                .partSpec("ds='" + outPart + "'")
                .label("main")
                .build());
        job.addOutput(TableInfo.builder()
                .projectName("winhc_ng")
                .tableName(outputRelationTab)
                .partSpec("ds='" + outPart + "'")
                .label("relation")
                .build());


        long startTime = System.currentTimeMillis();
        job.run();
        System.out.println("Job Finished in "
                + (System.currentTimeMillis() - startTime) / 1000.0 + " seconds");


    }
}
