package com.winhc.max.compute.graph.job.enterprise_group;


import com.aliyun.odps.conf.Configuration;
import com.aliyun.odps.io.Text;
import com.winhc.max.compute.graph.util.BaseUtils;

import java.util.Arrays;
import java.util.Set;
import java.util.stream.Collectors;

/**
 * @author: XuJiakai
 * 2023/3/1 17:05
 */
public class ProdEnvDebug {

    private static class WinhcDebugInfo extends RuntimeException {
        public WinhcDebugInfo() {
        }

        public WinhcDebugInfo(String message) {
            super(message);
        }

        public WinhcDebugInfo(String message, Throwable cause) {
            super(message, cause);
        }

        public WinhcDebugInfo(Throwable cause) {
            super(cause);
        }

        public WinhcDebugInfo(String message, Throwable cause, boolean enableSuppression, boolean writableStackTrace) {
            super(message, cause, enableSuppression, writableStackTrace);
        }
    }

    private boolean enableDebug;

    /**
     * debug位置
     */
    private Set<String> debugIndices;

    /**
     * debug 顶点id
     */
    private String debugVertexId;


    public static ProdEnvDebug build(Configuration configuration) {
        ProdEnvDebug prodEnvDebug = new ProdEnvDebug();
        prodEnvDebug.enableDebug = Boolean.parseBoolean(configuration.get("winhc.debug", "false"));
        String debugIndex = configuration.get("winhc.debug.index", null);
        if (debugIndex != null) {
            prodEnvDebug.debugIndices = Arrays.stream(debugIndex.split(",")).collect(Collectors.toSet());
        }
        prodEnvDebug.debugVertexId = configuration.get("winhc.debug.debugVertexId", null);
        return prodEnvDebug;
    }

    public void debug(Text vertexId, String template, Object... obj) {
        if (!enableDebug) {
            return;
        }
        String str = BaseUtils.format(template, obj);
        if (vertexId.toString().equals(debugVertexId)) {
            throw new WinhcDebugInfo(BaseUtils.format("\nvertex id: {}\n{}\n", vertexId, str));
        }
    }

    public void debugByIndex(String index, Text vertexId, String template, Object... obj) {
        if (!enableDebug) {
            return;
        }
        String str = BaseUtils.format(template, obj);
        String format = BaseUtils.format("\ndebug index: {}  vertex id: {}\n{}\n", index, vertexId, str);
        if (debugIndices == null || debugIndices.isEmpty()) {
            throw new WinhcDebugInfo(format);
        }
        if (vertexId.toString().equals(debugVertexId) && debugIndices.contains(index)) {
            throw new WinhcDebugInfo(format);
        }
    }

    public void debug_1(Text vertexId, String template, Object... obj) {
        debugByIndex("1", vertexId, template, obj);
    }

    public void debug_2(Text vertexId, String template, Object... obj) {
        debugByIndex("2", vertexId, template, obj);
    }

    public void debug_3(Text vertexId, String template, Object... obj) {
        debugByIndex("3", vertexId, template, obj);
    }


    public static void debugByCondition(boolean condition, String template, Object... obj) {
        if (condition) {
            throw new WinhcDebugInfo(BaseUtils.format(template, obj));
        }
    }

}
