package com.winhc.max.compute.graph.job.enterprise_group.entity;

import com.aliyun.odps.io.*;
import com.winhc.max.compute.graph.job.enterprise_group.EnterpriseGroupUtils;
import com.winhc.max.compute.graph.job.enterprise_group.EnterpriseGroupVertex;
import lombok.experimental.ExtensionMethod;

import java.io.DataInput;
import java.io.DataOutput;
import java.io.IOException;
import java.util.List;

/**
 * @author: XuJiakai
 * 2023/2/7 11:13
 */
@ExtensionMethod({
        EnterpriseGroupUtils.class
})
public class EntGroupMsg implements Writable {

    /**
     * 消息类型，1控股企业，2参股企业
     * todo:目前可以考虑不做type区分
     * {@link EnterpriseGroupVertex:90}
     */
    private IntWritable msgType;

    /**
     * 企业集团成员数
     */
    private LongWritable holdNum;

    /**
     * 下属控企业id,数组
     * 企业集团成员的id
     */
    private Tuple holdCompanyIds;

    /**
     * 集团对外投资id
     */
    private Tuple groupInvestmentCompanyIds;

    /**
     * 集团股东id，集团股东的股东是否算在内？不计算
     */
    private Tuple groupHolderCompanyIds;


    /**
     * 该条消息的首发节点信息，如果是名字包含集团才记录
     */
    private Text sourceVertexId;
    private Text sourceVertexName;


    /**
     * 途径id记录，防止陷入循环
     */
    private Tuple routeLog;


    /**
     * 股权链： (a)<-[0.51]-(b)<-[1.0]-(c)
     * 多个用逗号分割
     */
    private Text stockRightControlChain;


    public boolean addStockRightControlChain(String companyId, double investmentProportion, String holderKeyno) {
        String s = stockRightControlChain.toString();
        if (s.length() == 0) {
            stockRightControlChain.set("(" + companyId + ")<-[" + investmentProportion + "]-(" + holderKeyno + ")");
        } else {
            int lastIndexOf = s.lastIndexOf("(");
            String lastHolderId = s.substring(lastIndexOf + 1, s.length() - 1);
            if (companyId.equals(lastHolderId)) {
                s += "<-[" + investmentProportion + "]-(" + holderKeyno + ")";
                stockRightControlChain.set(s);
            } else {
                return false;
            }
        }
        return true;
    }


    /**
     * 记录并测试是否重复
     *
     * @param vertexId
     * @return false表示该节点已经走过。
     */
    public boolean routeLog(Text vertexId) {
        return routeLog(vertexId.toString());
    }

    public boolean routeLog(String vertexId) {
        long count = routeLog.getAll().stream().map(Object::toString).filter(e -> e.equals(vertexId)).count();

        if (count > 0) {
//            System.out.println("陷入循环，" + String.join(",", routeLog.tuple2List()));
            return false;
        }
        routeLog.append(new Text(vertexId));
        return true;


    }


    public static EntGroupMsg ofByType_1(Text thisVertexCompanyId, Text thisVertexCompanyName, IntWritable companyType, List<String> groupHolderCompanyIds) {
        //发给控股企业消息
        EntGroupMsg entGroupMsg = new EntGroupMsg();
        entGroupMsg.msgType = new IntWritable(1);
        entGroupMsg.holdNum = new LongWritable(1);
        entGroupMsg.holdCompanyIds = new Tuple(1);
        entGroupMsg.groupHolderCompanyIds = new Tuple();

        entGroupMsg.holdCompanyIds.append(thisVertexCompanyId);
        if (groupHolderCompanyIds != null) {
            for (String s : groupHolderCompanyIds) {
                entGroupMsg.groupHolderCompanyIds.append(new Text(s));
            }
        }

        entGroupMsg.addOtherMsg(thisVertexCompanyId, thisVertexCompanyName, companyType.get());
        return entGroupMsg;
    }


    public static EntGroupMsg ofByType_2(Text thisVertexCompanyId, Text thisVertexCompanyName) {
        //发给参股企业消息
        EntGroupMsg entGroupMsg = new EntGroupMsg();
        entGroupMsg.msgType = new IntWritable(2);
        entGroupMsg.holdNum = new LongWritable(0);
        entGroupMsg.groupInvestmentCompanyIds = new Tuple(1);
        entGroupMsg.groupInvestmentCompanyIds.append(new Text(thisVertexCompanyId));

        entGroupMsg.addOtherMsg(thisVertexCompanyId, null, null);
        return entGroupMsg;
    }


    private void addOtherMsg(Text thisVertexCompanyId, Text thisVertexCompanyName, Integer companyType) {
        if (thisVertexCompanyName != null && thisVertexCompanyId != null && companyType != null) {
            if (thisVertexCompanyName.toString().contains("集团") && companyType != 2) {
                this.sourceVertexId = thisVertexCompanyId;
                this.sourceVertexName = thisVertexCompanyName;
            }
        }

        this.routeLog = new Tuple();
        this.routeLog.append(thisVertexCompanyId);
        this.stockRightControlChain = new Text();
    }


    public Text getSourceVertexId() {
        return sourceVertexId;
    }

    public Text getSourceVertexName() {
        return sourceVertexName;
    }

    public int getMsgType() {
        return msgType.get();
    }

    public LongWritable getHoldNum() {
        return holdNum;
    }

    public Tuple getHoldCompanyIds() {
        return holdCompanyIds;
    }

    public Tuple getGroupInvestmentCompanyIds() {
        return groupInvestmentCompanyIds;
    }

    public Tuple getGroupHolderCompanyIds() {
        return groupHolderCompanyIds;
    }

    public Text getStockRightControlChain() {
        return stockRightControlChain;
    }

    @Override
    public void write(DataOutput out) throws IOException {
        if (holdCompanyIds == null) {
            holdCompanyIds = new Tuple();
        }

        if (groupInvestmentCompanyIds == null) {
            groupInvestmentCompanyIds = new Tuple();
        }

        if (groupHolderCompanyIds == null) {
            groupHolderCompanyIds = new Tuple();
        }
        if (routeLog == null) {
            routeLog = new Tuple();
        }
        if (stockRightControlChain == null)
            stockRightControlChain = new Text();

        if (sourceVertexId == null)
            sourceVertexId = new Text();
        if (sourceVertexName == null) {
            sourceVertexName = new Text();
        }

        msgType.write(out);
        holdNum.write(out);
        holdCompanyIds.write(out);
        groupInvestmentCompanyIds.write(out);
        groupHolderCompanyIds.write(out);
        routeLog.write(out);
        stockRightControlChain.write(out);
        this.sourceVertexId.write(out);
        this.sourceVertexName.write(out);
    }

    @Override
    public void readFields(DataInput in) throws IOException {

        msgType = new IntWritable();
        holdNum = new LongWritable();
        holdCompanyIds = new Tuple();
        groupInvestmentCompanyIds = new Tuple();
        groupHolderCompanyIds = new Tuple();
        routeLog = new Tuple();
        stockRightControlChain = new Text();

        sourceVertexId = new Text();
        sourceVertexName = new Text();


        msgType.readFields(in);
        holdNum.readFields(in);
        holdCompanyIds.readFields(in);
        groupInvestmentCompanyIds.readFields(in);
        groupHolderCompanyIds.readFields(in);
        routeLog.readFields(in);
        stockRightControlChain.readFields(in);
        this.sourceVertexId.readFields(in);
        this.sourceVertexName.readFields(in);
    }

    @Override
    public String toString() {
        return "EntGroupMsg{" +
                "msgType=" + msgType +
                ", holdNum=" + holdNum +
                ", holdCompanyIds=" + holdCompanyIds +
                ", groupInvestmentCompanyIds=" + groupInvestmentCompanyIds +
                ", groupHolderCompanyIds=" + groupHolderCompanyIds +
                ", sourceVertexId=" + sourceVertexId +
                ", sourceVertexName=" + sourceVertexName +
                ", routeLog=" + routeLog +
                ", stockRightControlChain=" + stockRightControlChain +
                '}';
    }
}
