package com.winhc.max.compute.graph.job.enterprise_group.entity;

import com.aliyun.odps.io.*;
import com.winhc.max.compute.graph.job.enterprise_group.EnterpriseGroupUtils;
import lombok.experimental.ExtensionMethod;

import java.io.DataInput;
import java.io.DataOutput;
import java.io.IOException;
import java.util.List;

/**
 * @author: XuJiakai
 * 2023/2/6 14:25
 * <p>
 * 企业集团顶点，公司或人
 */
@ExtensionMethod({
        EnterpriseGroupUtils.class
})
public class EnterpriseGroupVertexValue implements Writable {

    private Text companyName;
    /**
     * 公司类型，1为正常企业，2为分公司，3为合伙，4为机关单位，5为事业单位，6为人员节点
     */
    private IntWritable companyType;
    /**
     * 执行事务合伙人id,数组
     */
    private Tuple legalEntityIds;

    /**
     * 手动停止标记，需要手动拆分集团，第二次计算时才需要
     */
    private BooleanWritable manualStop;

    //以上为初始化信息，以下是计算相关信息

    /**
     * 子集团id
     */
    private Tuple subgroup;


    /**
     * 下属控股企业数
     * 企业集团成员数
     */
    private LongWritable holdNum;

    /**
     * 下属控企业id
     * 企业集团的id
     */
    private Tuple holdCompanyIds;

    /**
     * 集团对外投资id
     */
    private Tuple groupInvestmentCompanyIds;

    /**
     * 集团股东id
     * todo:目前该部分包括人
     */
    private Tuple groupHolderCompanyIds;

    /**
     * 集团控制人
     */
    private Text groupControllerKeyno;

    /**
     * 集团股权链
     */
    private Text stockRightControlChain;


    /**
     * 为true表示该节点发送过，只需要传递就ok
     * 0:未处理过，1：已经发送消息，2：集团节点，可以输出
     */
    private IntWritable endFlag;

    public void setEndFlag() {
        endFlag = new IntWritable(1);
    }

    public boolean isSendMsg() {
        return endFlag.get() != 0;
    }

    public void setOutput() {
        //由于执行顺序问题，只会从1 -> 2，不会从0 -> 2
        endFlag = new IntWritable(2);
    }

    public boolean isOutput() {
        return endFlag.get() == 2;
    }


    public IntWritable getCompanyType() {
        return companyType;
    }

    public Text getStockRightControlChain() {
        return stockRightControlChain;
    }

    public void setStockRightControlChain(Text stockRightControlChain) {
        this.stockRightControlChain = stockRightControlChain;
    }

    public BooleanWritable getManualStop() {
        return manualStop;
    }

    public void setSubgroup(Tuple subgroup) {
        this.subgroup = subgroup;
    }

    public Tuple getSubgroup() {
        return subgroup;
    }

    public Text getCompanyName() {
        return companyName;
    }

    public LongWritable getHoldNum() {
        return holdNum;
    }

    public Tuple getHoldCompanyIds() {
        return holdCompanyIds;
    }

    public Tuple getGroupInvestmentCompanyIds() {
        return groupInvestmentCompanyIds;
    }

    public Tuple getGroupHolderCompanyIds() {
        return groupHolderCompanyIds;
    }

    public Text getGroupControllerKeyno() {
        return groupControllerKeyno;
    }

    public void setHoldNum(long holdNum) {
        this.holdNum = new LongWritable(holdNum);
    }

    public void setHoldCompanyIds(List<String> holdCompanyIds) {
        this.holdCompanyIds = holdCompanyIds.list2Tuple();
    }

    public void setGroupInvestmentCompanyIds(List<String> groupInvestmentCompanyIds) {
        this.groupInvestmentCompanyIds = groupInvestmentCompanyIds.list2Tuple();
    }

    public void setGroupHolderCompanyIds(List<String> groupHolderCompanyIds) {
        this.groupHolderCompanyIds = groupHolderCompanyIds.list2Tuple();
    }

    public Tuple getLegalEntityIds() {
        return legalEntityIds;
    }

    public void setGroupControllerKeyno(Text groupControllerKeyno) {
        this.groupControllerKeyno = groupControllerKeyno;
    }

    public static EnterpriseGroupVertexValue of(String companyName) {
        return of(companyName, null, null, null);
    }

    public static EnterpriseGroupVertexValue of(String companyName, List<String> legalEntityIds) {
        return of(companyName, null, legalEntityIds, null);
    }

    public static EnterpriseGroupVertexValue of(String companyName, Integer companyType, List<String> legalEntityIds, Boolean manualStop) {
        EnterpriseGroupVertexValue enterpriseGroupVertexValue = new EnterpriseGroupVertexValue();
        enterpriseGroupVertexValue.companyName = new Text(companyName);
        if (companyType != null) {
            enterpriseGroupVertexValue.companyType = new IntWritable(companyType);
        } else {
            enterpriseGroupVertexValue.companyType = new IntWritable(1);
        }

        if (legalEntityIds != null && !legalEntityIds.isEmpty()) {
            enterpriseGroupVertexValue.legalEntityIds = new Tuple(legalEntityIds.size());
            enterpriseGroupVertexValue.companyType = new IntWritable(3);
            for (String legalEntityId : legalEntityIds) {
                Text text = new Text(legalEntityId);
                enterpriseGroupVertexValue.legalEntityIds.append(text);
            }
        }
        if (manualStop != null) {
            enterpriseGroupVertexValue.manualStop = new BooleanWritable(manualStop);
        } else {
            enterpriseGroupVertexValue.manualStop = new BooleanWritable(false);
        }
        enterpriseGroupVertexValue.fieldComplete();
        return enterpriseGroupVertexValue;
    }


    private void fieldComplete() {
        if (holdNum == null)
            holdNum = new LongWritable(0);

        if (holdCompanyIds == null)
            holdCompanyIds = new Tuple();

        if (groupInvestmentCompanyIds == null)
            groupInvestmentCompanyIds = new Tuple();

        if (groupHolderCompanyIds == null)
            groupHolderCompanyIds = new Tuple();

        if (groupControllerKeyno == null)
            groupControllerKeyno = new Text();

        if (endFlag == null)
            endFlag = new IntWritable(0);

        if (stockRightControlChain == null)
            stockRightControlChain = new Text();

        if (legalEntityIds == null)
            legalEntityIds = new Tuple();

        if (legalEntityIds == null) {
            manualStop = new BooleanWritable();
        }

        if (subgroup == null) {
            subgroup = new Tuple();
        }
    }


    @Override
    public void write(DataOutput out) throws IOException {
        companyName.write(out);
        companyType.write(out);
        legalEntityIds.write(out);
        manualStop.write(out);
        subgroup.write(out);
        holdNum.write(out);
        holdCompanyIds.write(out);
        groupInvestmentCompanyIds.write(out);
        groupHolderCompanyIds.write(out);
        groupControllerKeyno.write(out);
        stockRightControlChain.write(out);
        endFlag.write(out);
    }

    @Override
    public void readFields(DataInput in) throws IOException {
        companyName = new Text();
        companyType = new IntWritable();
        legalEntityIds = new Tuple();
        manualStop = new BooleanWritable();
        subgroup = new Tuple();
        holdNum = new LongWritable();
        holdCompanyIds = new Tuple();
        groupInvestmentCompanyIds = new Tuple();
        groupHolderCompanyIds = new Tuple();
        groupControllerKeyno = new Text();
        stockRightControlChain = new Text();
        endFlag = new IntWritable();

        companyName.readFields(in);
        companyType.readFields(in);
        legalEntityIds.readFields(in);
        manualStop.readFields(in);
        subgroup.readFields(in);
        holdNum.readFields(in);
        holdCompanyIds.readFields(in);
        groupInvestmentCompanyIds.readFields(in);
        groupHolderCompanyIds.readFields(in);
        groupControllerKeyno.readFields(in);
        stockRightControlChain.readFields(in);
        endFlag.readFields(in);
    }


    @Override
    public String toString() {
        return "EnterpriseGroupVertexValue{" +
                "companyName=" + companyName +
                ", companyType=" + companyType +
                ", legalEntityIds=" + legalEntityIds +
                ", manualStop=" + manualStop +
                ", subgroup=" + subgroup +
                ", holdNum=" + holdNum +
                ", holdCompanyIds=" + holdCompanyIds +
                ", groupInvestmentCompanyIds=" + groupInvestmentCompanyIds +
                ", groupHolderCompanyIds=" + groupHolderCompanyIds +
                ", groupControllerKeyno=" + groupControllerKeyno +
                ", stockRightControlChain=" + stockRightControlChain +
                ", endFlag=" + endFlag +
                '}';
    }
}
