package com.winhc.max.compute.graph.job.pagerank;

import com.aliyun.odps.graph.Aggregator;
import com.aliyun.odps.graph.WorkerContext;
import com.aliyun.odps.io.DoubleWritable;

import java.io.IOException;

/**
 * @author: XuJiakai
 * 2022/6/7 15:50
 */
public class PageRankAggregator extends Aggregator<PageRankAggValue> {
    @Override
    public PageRankAggValue createInitialValue(WorkerContext context) throws IOException {
        return new PageRankAggValue();
    }

    @Override
    public void aggregate(PageRankAggValue pageRankAggValue, Object item) throws IOException {
        DoubleWritable inputValue = ((DoubleWritable) item);
        pageRankAggValue.addNewValue(inputValue.get());
    }

    @Override
    public void merge(PageRankAggValue pageRankAggValue, PageRankAggValue partial) throws IOException {
        long n = pageRankAggValue.getN().get() + partial.getN().get();
        double sumValue = pageRankAggValue.getSumValue().get() + partial.getSumValue().get();
        pageRankAggValue.update(n, sumValue);
    }

    @Override
    public boolean terminate(WorkerContext context, PageRankAggValue pageRankAggValue) throws IOException {
        Double variance = Double.parseDouble(context.getConfiguration().get("company.rank.variance", "0.01"));
        double v = pageRankAggValue.getSumValue().get() / pageRankAggValue.getN().get();
        if (context.getSuperstep() == 0) {
            System.out.println("skip step " + context.getSuperstep() + ", variance value :" + v);
            return false;
        }
        System.out.println("step " + context.getSuperstep() + ", variance value: " + v);
        return v < variance;
    }
}
